from geometry_msgs.msg import Point, Pose, Twist
from trajectory_msgs.msg import JointTrajectory, JointTrajectoryPoint
from builtin_interfaces.msg import Duration
import rclpy
from rclpy.node import Node

from hexapod_python.ik_publisher_node import IKPublisherNode
from hexapod_python.ik import ik
from hexapod_python.ik_poses import calibration_pose

import copy

class IKCalibrationNode(IKPublisherNode):
    def __init__(self):
            super().__init__('ik_calibration')
            self.modified_calibration_pose = copy.deepcopy(calibration_pose)
            for leg in self.modified_calibration_pose['legs']:
                 self.modified_calibration_pose['legs'][leg].z += 20
            
            self.calibration_leg='e6'
            
            [ self.ik_values_reference ] = [ ik(calibration_pose) ]            
            ik_values = [ ik(self.modified_calibration_pose)]



            self.publish_ik_values(ik_values)

            self.cmd_vel_subscription = self.create_subscription(
                 Twist,
                 "/cmd_vel",
                 self.twist_callback,
                 10                
            )
    
    def twist_callback(self, msg):
        self.modified_calibration_pose['legs'][self.calibration_leg].x -= msg.linear.x
        self.modified_calibration_pose['legs'][self.calibration_leg].y -= msg.linear.y
        self.modified_calibration_pose['legs'][self.calibration_leg].z -= msg.linear.z
        ik_values_list = [ ik(self.modified_calibration_pose)]
        [ ik_values ] = ik_values_list
        
        im = ik_values[self.calibration_leg]
        ir = self.ik_values_reference[self.calibration_leg]

        print(im)
        print(ir)
        print([(m - r) for (m,r) in zip(im, ir)])
        print([(m - r) * 129 for (m,r) in zip(im, ir)])
        print()

        self.publish_ik_values(ik_values_list)


def main(args = None):
    rclpy.init(args=args)

    ik_publisher = IKCalibrationNode()
    rclpy.spin(ik_publisher)
    ik_publisher.destroy_node()
    rclpy.shutdown()
    