import rclpy
from rclpy.node import Node

from hexapod_python.ik_publisher_node import IKPublisherNode
from hexapod_python.ik import ik
from hexapod_python.ik_poses import interpolate_pose, standing_pose, stand_up_pose, shutdown_pose

import time
import copy


class IKTestNode(IKPublisherNode):
    def __init__(self):
            super().__init__('ik_test')
            ik_values = [ik(pose) for pose in interpolate_pose(shutdown_pose, stand_up_pose, 50)]
            self.publish_ik_values(ik_values)
            time.sleep(1)

            current_pose = copy.deepcopy(stand_up_pose)

            ik_values = []
            for leg in self.leg_groups_:
                intermediate_pose_1 = copy.deepcopy(current_pose)
                intermediate_pose_1['legs'][leg].z += 20

                intermediate_pose_2 = copy.deepcopy(intermediate_pose_1)
                intermediate_pose_2['legs'][leg] = copy.deepcopy(standing_pose['legs'][leg])
                intermediate_pose_2['legs'][leg].z += 20

                intermediate_pose_3 = copy.deepcopy(intermediate_pose_2)
                intermediate_pose_3['legs'][leg] = copy.deepcopy(standing_pose['legs'][leg])

                ik_values += (
                    [ik(pose) for pose in interpolate_pose(current_pose, intermediate_pose_1, 10)]+
                    [ik(pose) for pose in interpolate_pose(intermediate_pose_1, intermediate_pose_2, 20)] +
                    [ik(pose) for pose in interpolate_pose(intermediate_pose_2, intermediate_pose_3, 10)]
                )
                current_pose = copy.deepcopy(intermediate_pose_3)

            self.publish_ik_values(ik_values)
            time.sleep(8)

            current_pose = copy.deepcopy(standing_pose)
            ik_values = []
            for leg in self.leg_groups_:
                intermediate_pose_1 = copy.deepcopy(current_pose)
                intermediate_pose_1['legs'][leg].z += 20

                intermediate_pose_2 = copy.deepcopy(intermediate_pose_1)
                intermediate_pose_2['legs'][leg] = copy.deepcopy(stand_up_pose['legs'][leg])
                intermediate_pose_2['legs'][leg].z += 20

                intermediate_pose_3 = copy.deepcopy(intermediate_pose_2)
                intermediate_pose_3['legs'][leg] = copy.deepcopy(stand_up_pose['legs'][leg])

                ik_values += (
                    [ik(pose) for pose in interpolate_pose(current_pose, intermediate_pose_1, 10)]+
                    [ik(pose) for pose in interpolate_pose(intermediate_pose_1, intermediate_pose_2, 20)] +
                    [ik(pose) for pose in interpolate_pose(intermediate_pose_2, intermediate_pose_3, 10)]
                )
                current_pose = copy.deepcopy(intermediate_pose_3)

            self.publish_ik_values(ik_values)
            time.sleep(6)


            ik_values = [ik(pose) for pose in interpolate_pose(stand_up_pose, shutdown_pose, 50)]
            self.publish_ik_values(ik_values)
            raise SystemExit

def main(args = None):
    rclpy.init(args=args)

    ik_publisher = IKTestNode()
    rclpy.spin_once(ik_publisher)
    ik_publisher.destroy_node()
    rclpy.shutdown()
    